VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "SimpleExecutive"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' John Robbins
' Microsoft Systems Journal - August, 1997
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FILE          :   SimpleExecutive.cls
' DESCRIPTION   :
'  Implements a simple debugger executive that conforms to the
'  BaseExecutive abstract base class.  If you want to extend VBDebug to
'  do handle more advanced things, derive your own class from
'  BaseExecutive.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Polymorphic Bliss
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Implements BaseExecutive

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'Class Specific Private Variables
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' The handle to the main thread.
Private g_hMainThread As Long
' The handle to the process.
Private g_hProcess As Long
' Has the initial breakpoint already been seen?
Private g_bSeenFirstBP As Boolean
' The synchronization object that keeps us from having trouble in this
'  class.
Private g_clsCritSec As CriticalSection
' The text box where all output is placed.
Private g_txtOutput As TextBox

' The internal list of threads.
Private g_colThreads As Collection
' The internal list of DLLs.
Private g_colDlls As Collection

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'Property Setting Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Friend Property Let txtOutput(txtBox As TextBox)
   Set g_txtOutput = txtBox
End Property

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : Class_Initialize
' DISCUSSION      :
'  The initialization for the class.
' PARAMETERS      :
'  None.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Private Sub Class_Initialize()
   Set g_clsCritSec = New CriticalSection
   Set g_colThreads = New Collection
   Set g_colDlls = New Collection
   g_bSeenFirstBP = False
End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : Class_Terminate
' DISCUSSION      :
'  The termination for the class.
' PARAMETERS      :
'  None.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Private Sub Class_Terminate()
   Set g_clsCritSec = Nothing
   Set g_colThreads = Nothing
   Set g_colDlls = Nothing
End Sub

Private Sub BaseExecutive_DebugCreateProcess _
                                        (clsDebugger As DebuggerClass, _
                                         dwProcessID As Long, _
                                         dwThreadID As Long)
   g_clsCritSec.Enter
   On Error GoTo DebugCreateProcess_Error

   g_hMainThread = clsDebugger.CreateProcessDbgEvt.hThread
   g_hProcess = clsDebugger.CreateProcessDbgEvt.hProcess

#If DEBUGBUILD Then
   OutputDebugString ("Created Process ID: " + _
                      Hex$(dwProcessID) + _
                      " Thread Handle : " + _
                      Hex$(g_hProcess) + _
                      " With Thread " + _
                      Hex$(g_hMainThread) + _
                      vbNewLine)
#End If

   AppendText "Process &H" + Hex$(g_hProcess) + " created"
   AppendText "Thread &H" + Hex$(g_hMainThread) + " created"

   ' Add the main thread to the collection.
   g_colThreads.Add CStr(g_hMainThread), CStr(dwThreadID)

#If DEBUGBUILD Then
   OutputDebugString ("Adding thread &H" + _
                        Hex$(g_hMainThread) + _
                        " to thread collection" + _
                        vbNewLine)
   OutputDebugString ("Total of " + _
                      CStr(g_colThreads.Count) + _
                      " in the collection" + vbNewLine)
#End If

   g_clsCritSec.Leave
   Exit Sub

DebugCreateProcess_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugCreateProcess Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_DebugCreateThread _
                                        (clsDebugger As DebuggerClass, _
                                         dwProcessID As Long, _
                                         dwThreadID As Long)
   g_clsCritSec.Enter
   On Error GoTo DebugCreateThread_Error

   Dim stCTDI As CREATE_THREAD_DEBUG_INFO
   stCTDI = clsDebugger.CreateThreadDbgEvt

   g_colThreads.Add CStr(stCTDI.hThread), CStr(dwThreadID)

   AppendText "Thread &H" + Hex$(stCTDI.hThread) + " created"

#If DEBUGBUILD Then
   OutputDebugString ("Adding thread &H" + _
                      Hex$(stCTDI.hThread) + _
                      " to thread collection" + vbNewLine)
   OutputDebugString ("Total of " + _
                      CStr(g_colThreads.Count) + _
                      " in the collection" + vbNewLine)
#End If

   g_clsCritSec.Leave
   Exit Sub

DebugCreateThread_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugCreateThread Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_DebugDllLoad(clsDebugger As DebuggerClass, _
                                       dwProcessID As Long, _
                                       dwThreadID As Long)
   g_clsCritSec.Enter
   On Error GoTo DebugDllLoad_Error

   Dim stLDDI As LOAD_DLL_DEBUG_INFO
   stLDDI = clsDebugger.LoadDllDbgEvt

   g_colDlls.Add Hex$(stLDDI.lpBaseOfDll), Hex$(stLDDI.lpBaseOfDll)
   AppendText "DLL Loaded at &H" + Hex$(stLDDI.lpBaseOfDll)

#If DEBUGBUILD Then
   OutputDebugString ("Adding DLL &H" + _
                        Hex$(stLDDI.lpBaseOfDll) + _
                        " to DLL collection" + _
                        vbNewLine)
   OutputDebugString ("Total of " + _
                      CStr(g_colDlls.Count) + _
                      " in the collection" + vbNewLine)
#End If

   g_clsCritSec.Leave
   Exit Sub

DebugDllLoad_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugDllLoad Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_DebugDllUnload(clsDebugger As DebuggerClass, _
                                         dwProcessID As Long, _
                                         dwThreadID As Long)
   g_clsCritSec.Enter
   On Error GoTo DebugDllUnload_Error

   Dim stULDDI As UNLOAD_DLL_DEBUG_INFO
   stULDDI = clsDebugger.UnloadDllDbgEvt

   g_colDlls.Remove Hex$(stULDDI.lpBaseOfDll)

   AppendText "DLL Unloaded at &H" + Hex$(stULDDI.lpBaseOfDll)

#If DEBUGBUILD Then
   OutputDebugString ("Removing DLL &H" + _
                        Hex$(stULDDI.lpBaseOfDll) + _
                        " from the DLL collection" + _
                        vbNewLine)
   OutputDebugString ("Total of " + _
                      CStr(g_colDlls.Count) + _
                      " in the collection" + vbNewLine)
#End If

   g_clsCritSec.Leave
   Exit Sub

DebugDllUnload_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugDllUnload Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_DebugException(clsDebugger As DebuggerClass, _
                                         dwContType As Long, _
                                         dwProcessID As Long, _
                                         dwThreadID As Long)
   g_clsCritSec.Enter
   On Error GoTo DebugException_Error

   Dim stER As EXCEPTION_RECORD
   stER = clsDebugger.ExceptionDbgEvt.ExceptionRecord

   ' If this is a breakpoint exception and if the loader breakpoint has
   '  not been seen yet, then it is OK.
   If ((False = g_bSeenFirstBP) And _
       (EXCEPTION_BREAKPOINT = stER.ExceptionCode)) Then

      g_bSeenFirstBP = True
#If ALPHA Then
      SkipBreakPoint g_hMainThread
#End If
      dwContType = DBG_CONTINUE

   Else

      ' The program has a problem.  Here is where stack dumps and other
      '  helpful things could occur.
      ' If this is a first chance exception, pass it on to the debuggee
      '  to let them handle it first.  The second time that it is seen,
      '  show the warning.
      If (0 = clsDebugger.ExceptionDbgEvt.dwFirstChance) Then

         AppendText GetExceptionString(stER.ExceptionCode) + _
                    " occurred at &H" + _
                    Hex$(stER.ExceptionAddress)
      End If

      dwContType = DBG_EXCEPTION_NOT_HANDLED

   End If

   g_clsCritSec.Leave
   Exit Sub

DebugException_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugException Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_DebugExitProcess _
                                       (clsDebugger As DebuggerClass, _
                                        dwProcessID As Long, _
                                        dwThreadID As Long)
   g_clsCritSec.Enter
   On Error GoTo DebugExitProcess_Error

#If DEBUGBUILD Then
   Dim hThread As Long
   hThread = g_colThreads.Item(CStr(dwThreadID))
#End If

   g_colThreads.Remove CStr(dwThreadID)

   AppendText "Process &H" + _
               Hex$(g_hProcess) + _
               " ended and returned &H" + _
               Hex$(clsDebugger.ExitProcessDbgEvt.dwExitCode)

#If DEBUGBUILD Then
   OutputDebugString ("Removing thread &H" + _
                        Hex$(hThread) + _
                        " from the thread collection" + _
                        vbNewLine)
   OutputDebugString ("Total of " + _
                      CStr(g_colThreads.Count) + _
                      " in the collection" + vbNewLine)
#End If

   g_clsCritSec.Leave
   Exit Sub

DebugExitProcess_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugExitProcess Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_DebugExitThread _
                                       (clsDebugger As DebuggerClass, _
                                        dwProcessID As Long, _
                                        dwThreadID As Long)
   g_clsCritSec.Enter
   On Error GoTo DebugExitThread_Error

   Dim hThread As Long
   hThread = g_colThreads.Item(CStr(dwThreadID))

   g_colThreads.Remove CStr(dwThreadID)

   AppendText "Thread &H" + _
              Hex$(hThread) + _
              " ended and returned &H" + _
              Hex$(clsDebugger.ExitThreadDbgEvt.dwExitCode)

#If DEBUGBUILD Then
   OutputDebugString ("Removing thread &H" + _
                        Hex$(hThread) + _
                        " from the thread collection" + _
                        vbNewLine)
   OutputDebugString ("Total of " + _
                      CStr(g_colThreads.Count) + _
                      " in the collection" + vbNewLine)
#End If

   g_clsCritSec.Leave
   Exit Sub

DebugExitThread_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugExitThread Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_DebugODS(clsDebugger As DebuggerClass, _
                                   dwProcessID As Long, _
                                   dwThreadID As Long)

   g_clsCritSec.Enter
   On Error GoTo DebugODS_Error

   Dim stODS As OUTPUT_DEBUG_STRING_INFO
   Dim szOutBuff As String * 1024
   Dim bRet As Long
   Dim dwBytes As Long

   stODS = clsDebugger.ODSDbgEvt

   bRet = ReadProcessMemory(g_hProcess, _
                            stODS.lpDebugStringData, _
                            szOutBuff, _
                            stODS.nDebugStringLength, _
                            dwBytes)

   AppendText "ODS:  " + Left$(szOutBuff, stODS.nDebugStringLength)

   g_clsCritSec.Leave
   Exit Sub

DebugODS_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugODS Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_DebugRipInfo(clsDebugger As DebuggerClass, _
                                       dwProcessID As Long, _
                                       dwThreadID As Long)

   g_clsCritSec.Enter
   On Error GoTo DebugRipInfo_Error

   g_clsCritSec.Leave
   Exit Sub

DebugRipInfo_Error:
#If DEBUGBUILD Then
   MsgBox ("DebugRipInfo Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_PauseProcess()

   g_clsCritSec.Enter
   On Error GoTo PauseProcess_Error

   Dim lData As Variant
   Dim bRet As Long

   For Each lData In g_colThreads
      bRet = SuspendThread(CLng(lData))
   Next lData

   g_clsCritSec.Leave
   Exit Sub

PauseProcess_Error:
#If DEBUGBUILD Then
   MsgBox ("PauseProcess Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

Private Sub BaseExecutive_ResumeProcess()

   g_clsCritSec.Enter
   On Error GoTo ResumeProcess_Error

   Dim lData As Variant
   Dim bRet As Long

   For Each lData In g_colThreads
      bRet = ResumeThread(CLng(lData))
   Next lData


   g_clsCritSec.Leave
   Exit Sub

ResumeProcess_Error:
#If DEBUGBUILD Then
   MsgBox ("ResumeProcess Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : AppendText
' DISCUSSION      :
'  This is a public interface off the SimpleExecutive class.  Both the
'  UI and this class will only call through it to do their output.
' PARAMETERS      :
'  szStr - The string to append.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Sub AppendText(szStr As String)

   ' Hey!  Shouldn't there be a critical section in here?  Technically,
   '  there should be, but since the edit control we are protecting is
   '  protected internally, it doesn't really need it.

   On Error GoTo AppendText_Error

   Dim lLen As Long
   lLen = Len(szStr)

   If (0 = lLen) Then
      Exit Sub
   End If

   Dim szTemp As String
   szTemp = g_txtOutput.Text
   If (Chr$(13) <> Right$(szStr, lLen - 2)) Or _
      (Chr$(10) <> Right$(szStr, lLen - 1)) Then
      g_txtOutput.Text = szTemp + szStr + vbNewLine
   Else
      g_txtOutput.Text = szTemp + szStr
   End If

   ' Now scroll the text into view.
   g_txtOutput.SelStart = Len(g_txtOutput.Text)

   Exit Sub

AppendText_Error:
#If DEBUGBUILD Then
   MsgBox ("AppendText Error: " + Err.Description)
#End If

End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : DumpActiveThreads
' DISCUSSION      :
'  To demonstrate some of the cross thread coordination, this function
'  dumps the active threads for the debuggee.  This is to give you an
'  idea how to access information from the UI.
' PARAMETERS      :
'  None.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Sub DumpActiveThreads()

   g_clsCritSec.Enter
   On Error GoTo PauseProcess_Error

   Dim lData As Variant
   Dim i As Long

   For Each lData In g_colThreads

      AppendText "Info:  Active thread #" + _
                  CStr(i) + _
                  " Handle : &H" + _
                  Hex$(CLng(lData))
      i = i + 1

   Next lData

   g_clsCritSec.Leave
   Exit Sub

PauseProcess_Error:
#If DEBUGBUILD Then
   MsgBox ("DumpActiveThreads Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : DumpLoadedDLLs
' DISCUSSION      :
'  Like DumpActiveThreads, dumps the loaded DLLs that are currently in
'  the debuggee's address space.
' PARAMETERS      :
'  None.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Sub DumpLoadedDLLs()

   g_clsCritSec.Enter
   On Error GoTo PauseProcess_Error

   Dim szData As Variant
   Dim i As Long

   For Each szData In g_colDlls

      AppendText "Info:  DLL #" + _
                  CStr(i) + _
                  " Loaded at &H" + _
                  szData
      i = i + 1

   Next szData

   g_clsCritSec.Leave
   Exit Sub

PauseProcess_Error:
#If DEBUGBUILD Then
   MsgBox ("DumpActiveThreads Error: " + Err.Description)
#End If
   g_clsCritSec.Leave

End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        :  SkipBreakPoint
' DISCUSSION      :
'  The function that skips over ALPHA breakpoints.  Where INTEL
'  breakpoints automatically increment EIP when hit, on the ALPHA, Fir
'  is not.
' PARAMETERS      :
'  hThread - The thread where the skip is to take place.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
#If ALPHA Then
Private Sub SkipBreakPoint(hThread As Long)

   Dim ctx As CONTEXT
   Dim bRet As Long

   ctx.ContextFlags = CONTEXT_CONTROL

   bRet = GetThreadContext(hThread, ctx)
   If (1 <> bRet) Then
      MsgBox ("GetThreadContext failed!")
   End If

   ' ALPHA instructions are all four bytes long.
   ctx.Fir.lowpart = ctx.Fir.lowpart + 4

   bRet = SetThreadContext(hThread, ctx)

   If (1 <> bRet) Then
      MsgBox ("SetThreadContext failed!")
   End If

End Sub
#End If

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        :  GetExceptionString
' DISCUSSION      :
'  Given an exception code, returns the human readable string that
'  describes it.
' PARAMETERS      :
'  dwCode - The exception code.
' RETURN          :
'  The string.  If the exception is unknown then "Unknown exception"
'  is returned.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Private Function GetExceptionString(dwCode As Long) As String

   Select Case (dwCode)
      Case EXCEPTION_ACCESS_VIOLATION
         GetExceptionString = "Access Violation Exception"
         Exit Function
      Case EXCEPTION_DATATYPE_MISALIGNMENT
         GetExceptionString = "Datatype Misalignment Exception"
         Exit Function
      Case EXCEPTION_BREAKPOINT
         GetExceptionString = "Breakpoint Exception"
         Exit Function
      Case EXCEPTION_SINGLE_STEP
         GetExceptionString = "Single Step Exception"
         Exit Function
      Case EXCEPTION_ARRAY_BOUNDS_EXCEEDED
         GetExceptionString = "Array Bounds Exceeded Exception"
         Exit Function
      Case EXCEPTION_FLT_DENORMAL_OPERAND
         GetExceptionString = "Floating Point Denormal Operand Exception"
         Exit Function
      Case EXCEPTION_FLT_DIVIDE_BY_ZERO
         GetExceptionString = "Floating Point Divide By Zero Exception"
         Exit Function
      Case EXCEPTION_FLT_INEXACT_RESULT
         GetExceptionString = "Floating Point Inexact Result Exception"
         Exit Function
      Case EXCEPTION_FLT_INVALID_OPERATION
         GetExceptionString = "Floating Point Invalid Operation Exception"
         Exit Function
      Case EXCEPTION_FLT_OVERFLOW
         GetExceptionString = "Floating Point Overflow Exception"
         Exit Function
      Case EXCEPTION_INT_DIVIDE_BY_ZERO
         GetExceptionString = "Integer Divide By Zero Exception"
         Exit Function
      Case EXCEPTION_INT_OVERFLOW
         GetExceptionString = "Integer Overflow Exception"
         Exit Function
      Case EXCEPTION_PRIV_INSTRUCTION
         GetExceptionString = "Privileged Instruction Exception"
         Exit Function
      Case EXCEPTION_IN_PAGE_ERROR
         GetExceptionString = "In Page Error Exception"
         Exit Function
      Case EXCEPTION_ILLEGAL_INSTRUCTION
         GetExceptionString = "Illegal Instruction Exception"
         Exit Function
      Case EXCEPTION_NONCONTINUABLE_EXCEPTION
         GetExceptionString = "Noncontinuable Exception"
         Exit Function
      Case EXCEPTION_STACK_OVERFLOW
         GetExceptionString = "Stack Overflow Exception"
         Exit Function
      Case EXCEPTION_INVALID_DISPOSITION
         GetExceptionString = "Invalid Disposition Exception"
         Exit Function
      Case EXCEPTION_GUARD_PAGE
         GetExceptionString = "Guard Page Exception"
         Exit Function
      Case Else
         GetExceptionString = "Unknown Exception (&H" + _
                              Hex$(dwCode) + ")"
         Exit Function
   End Select

End Function
